import json
# import logging
import os

import pytest
import jsonschema

DEFAULT_REQUEST_HEADERS = {
    "Content-type": "application/json",
    "Accept": ["application/json"]
}

MODULE_DIR = os.path.realpath(os.path.join(
    os.path.dirname(__file__),
    "..",
    "inventory_provider"))


def test_version_request(client):
    version_schema = {
        "$schema": "http://json-schema.org/draft-07/schema#",
        "type": "object",
        "properties": {
            "api": {
                "type": "string",
                "pattern": r'\d+\.\d+'
            },
            "module": {
                "type": "string",
                "pattern": r'\d+\.\d+'
            }
        },
        "required": ["api", "module"],
        "additionalProperties": False
    }

    rv = client.post(
        "data/version",
        headers=DEFAULT_REQUEST_HEADERS)
    assert rv.status_code == 200
    jsonschema.validate(
        json.loads(rv.data.decode("utf-8")),
        version_schema)


class MockedRedis(object):

    db = None

    def __init__(self, *args, **kwargs):
        if MockedRedis.db is None:
            test_data_filename = os.path.join(
                os.path.dirname(__file__),
                "router-info.json")
            with open(test_data_filename) as f:
                MockedRedis.db = json.loads(f.read())

    def set(self, key, value):
        MockedRedis.db[key] = value

    def hget(self, key, field):
        value = MockedRedis.db[key]
        return json.dumps(value[field]).encode('utf-8')

    def hgetall(self, key):
        result = {}
        for k, v in MockedRedis.db[key].items():
            result[k.encode('utf-8')] \
                = json.dumps(v).encode('utf-8')
        return result

    def keys(self, *args, **kwargs):
        return list([k.encode("utf-8") for k in MockedRedis.db.keys()])


@pytest.fixture
def client_with_mocked_data(mocker, client):
    mocker.patch(
        'inventory_provider.router_details.redis.StrictRedis',
        MockedRedis)
    mocker.patch(
        'inventory_provider.routes.data.redis.StrictRedis',
        MockedRedis)
    return client


def _routers(client):
    routers_list_schema = {
        "$schema": "http://json-schema.org/draft-07/schema#",
        "type": "array",
        "items": {"type": "string"}
    }

    rv = client.post(
        "data/routers",
        headers=DEFAULT_REQUEST_HEADERS)
    assert rv.status_code == 200

    response = json.loads(rv.data.decode("utf-8"))
    jsonschema.validate(response, routers_list_schema)
    return response


def test_routers_list(client_with_mocked_data):
    assert _routers(client_with_mocked_data)


def test_router_interfaces(client_with_mocked_data):

    interfaces_list_schema = {
        "$schema": "http://json-schema.org/draft-07/schema#",
        "type": "array",
        "items": {
            "type": "object",
            "properties": {
                "name": {"type": "string"},
                "description": {"type": "string"}
            },
            "required": ["name", "description"],
            "additionalProperties": False
        }
    }

    for router in _routers(client_with_mocked_data):
        rv = client_with_mocked_data.post(
            "/data/interfaces/" + router,
            headers=DEFAULT_REQUEST_HEADERS)

        response = json.loads(rv.data.decode("utf-8"))
        jsonschema.validate(response, interfaces_list_schema)
        assert response  # at least shouldn't be empty


def test_snmp_ids(client_with_mocked_data):

    snmp_id_list_schema = {
        "$schema": "http://json-schema.org/draft-07/schema#",
        "type": "array",
        "items": {
            "type": "object",
            "properties": {
                "index": {"type": "string"},
                "name": {"type": "string"}
            },
            "required": ["index", "name"],
            "additionalProperties": False
        }
    }

    for hostname in _routers(client_with_mocked_data):
        rv = client_with_mocked_data.post(
            "/data/snmp/" + hostname,
            headers=DEFAULT_REQUEST_HEADERS)

        response = json.loads(rv.data.decode("utf-8"))
        jsonschema.validate(response, snmp_id_list_schema)
        assert response  # at least shouldn't be empty


def test_router_bgp_route(client_with_mocked_data):

    bgp_list_schema = {
        "$schema": "http://json-schema.org/draft-07/schema#",
        "type": "array",
        "items": {
            "type": "object",
            "properties": {
                "description": {"type": "string"},
                "as": {
                    "type": "object",
                    "properties": {
                        "peer": {
                            "type": "string",
                            "pattern": r'^\d+$'
                        },
                        "local": {
                            "type": "string",
                            "pattern": r'^\d+$'
                        },
                    },
                    "required": ["peer", "local"],
                    "additionalProperties": False
                },
            },
            "required": ["description", "as"],
            "additionalProperties": False
        }
    }

    routers_with_bpg_configs = [
        "mx1.mil2.it.geant.net",
        "mx1.vie.at.geant.net",
        "mx1.fra.de.geant.net",
        "mx1.ams.nl.geant.net",
        "mx1.pra.cz.geant.net",
        "mx1.dub.ie.geant.net",
        "mx1.mad.es.geant.net",
        "mx1.gen.ch.geant.net",
        "mx1.mar.fr.geant.net",
        "mx1.lon.uk.geant.net"
    ]

    for router in _routers(client_with_mocked_data):

        if router not in routers_with_bpg_configs:
            continue

        rv = client_with_mocked_data.post(
            "/data/bgp/" + router,
            headers=DEFAULT_REQUEST_HEADERS)

        response = json.loads(rv.data.decode("utf-8"))
        jsonschema.validate(response, bgp_list_schema)
        assert response  # at least shouldn't be empty


def test_router_debug_data_route(client_with_mocked_data):
    debug_data_schema = {
        "$schema": "http://json-schema.org/draft-07/schema#",
        "type": "object",
        "items": {
            "type": "object",
            "properties": {
                "vrr": {"type": "object"},
                "bgp": {"type": "array"},
                "interfaces": {"type": "array"}
            },
            "required": ["vrr", "bgp", "interfaces"],
            "additionalProperties": False
        }
    }

    for router in _routers(client_with_mocked_data):
        rv = client_with_mocked_data.post(
            "/data/debug-dump/" + router,
            headers=DEFAULT_REQUEST_HEADERS)
        response = json.loads(rv.data.decode("utf-8"))
        jsonschema.validate(response, debug_data_schema)
